/* __FreeRTOS_NeoMatrix_8x1_04.ino
    2022-03-30. Don Prefontaine
    1 row of 8 Neopixel LEDs. Code supports both strands and matrices of LEDs.
    FreeRTOS is used to multitask a speaker, a pushbutton switch, and a strand/matrix of neopixel LEDs.
    tone(): https://itp.nyu.edu/physcomp/labs/labs-arduino-digital-and-analog/tone-output-using-an-arduino/
    If using a 5v Arduino UNO, all pins are available. Other microcontrollers don't have that luxury.
     - Feather M4 Express pins: A0-A5 and D0-D13 except D2, D3, D7, D8.
     - QT-PY pins: A0,A1,A2,A3,SDA,SCL,A6. A7,A8,A9,A10,3v3,GND,5v.
    FreeRTOS setup: build within setup(), not loop().
         Objective: play a sound/tune and press a switch while displaying neopixel LEDs.
*/
#include <FreeRTOS_SAMD51.h>    // For Adafruit Feather M4 Express with SAMD51 uC.
//#include <FreeRTOS_SAMD21.h>  // For Adafruit QT-PY with SAMD21 uC.
//#include <Arduino_FreeRTOS.h> // For 5v Arduino UNO.
//#include <semphr.h>           // Add the FreeRTOS functions for Semaphores (or Flags).

void marchLR(void *pvParameters);     // Display LEDs sequentially while extinguishing the previous ones.
TaskHandle_t  marchLR_handle;

void beepUP(void *pvParameters);      // Play a brief tune of escalating tones.
TaskHandle_t  beepUP_handle;

void beepDOWN(void *pvParameters);    // Play a brief tune of de-escalating tones.
TaskHandle_t  beepDOWN_handle;

//void BEEPupDown(void *pvParameters);
//TaskHandle_t  BEEPupDown_handle;

void swBlue(void *pvParameters);      // Illuminate a blue LED when a momentary switch is pressed and released.
TaskHandle_t  swBlue_handle;          // De-illuminate the blue LED when the switch is activated a second time.
//-----------------------------------------------------------------------
#define RED           0xF800
#define GREEN         0x07E0
#define YELLOW        0xFFE0
#define BLUE          0x001F
#define SILVER        0xC618
#define VIOLET        0x8888
#define MAGENTA       0xF81F
#define DarkMAGENTA   0x612F
#define ORANGE        0xFD20
#define BLACK         0x0000
//-----------------------------------------------------------------------
#include <Adafruit_GFX.h>       // Graphics library for all displays and LEDs.
#include <Adafruit_NeoPixel.h>  // For NeoPixel LEDs.
#include <Adafruit_NeoMatrix.h> // For matrices of NeoPixels.
int RGB          = 4;           // (D4 on M4 Feather. A7 on QTPY.) Neopixel strand/matrix output pin.
int pushButton   = 5;           // (D5 on M4 Feather. A6 on QTPY.) Pushbutton pin.
int blueLED      = A2;          // (A2 on QTPY & M4 Feather.) Separate blue LED.
int speakerPin   = A3;          // (A3 on QTPY & M4 Feather.) Buzzer/speaker output pin.
int ledCnt = 8;                 // Number of neopixel LEDs.
int LEDstate   = 0;             // Is blue LED on or off?
int pushOLD    = 1;             // 0 = button pushed, 1 = button not pushed (pulled high).
int pushNEW    = 0;
// Width,Height,Pin,First Pixel Position,Rows or Columns,Progressive or Zigzag,GRB or RGB or RGBW,protocol speed.
Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(ledCnt, 1, 4,
                            NEO_MATRIX_TOP  + NEO_MATRIX_LEFT +
                            NEO_MATRIX_ROWS + NEO_MATRIX_PROGRESSIVE,
                            NEO_GRB         + NEO_KHZ800);            // GRB not RGB!
int aC[9] = {RED, GREEN, YELLOW, BLUE, SILVER, VIOLET, MAGENTA, ORANGE, BLACK}; // aC = arrayColours 0 to 8.
int i, j, k = 0;  // Initialize counters.
//************************************************************************//
//******************************* SETUP **********************************//
//************************************************************************//
void setup() {
  pinMode(speakerPin, OUTPUT);        // Buzzer, amplified buzzer or speaker is used.
  pinMode(pushButton, INPUT_PULLUP);  // Input push button for lighting blue LED.
  pinMode(blueLED, OUTPUT);           // Output pin for push button.
  Serial.begin(9600);                 // Optional for troubleshooting.
  matrix.begin();                     // Initialize the matrix library.
  matrix.setBrightness(40);           // 0-255.
  /* xTaskCreate section: create all of your tasks here. */
  xTaskCreate(marchLR, "March LtoR", 128, NULL, 1, &marchLR_handle);
  xTaskCreate(beepUP, "Beep Frequency Ascending", 128, NULL, 1, &beepUP_handle);       vTaskSuspend(beepUP_handle);
  xTaskCreate(beepDOWN, "Beep Frequency Descending", 128, NULL, 1, &beepDOWN_handle);  vTaskSuspend(beepDOWN_handle);
  xTaskCreate(swBlue, "Toggle Blue LED", 128, NULL, 1, &swBlue_handle);
  //xTaskCreate(BEEPupDown, "Beep Ascending & Descending", 128, NULL, 1, &BEEPupDown_handle);

  vTaskStartScheduler();  //Start the scheduler in setup().
}
//***********************************************************************//
void marchLR(void *pvParameters) { // Left-Right March. Display first Neo LED briefly then advance to next Neo LED, left to right.
  (void) pvParameters;
  while (1) {                      // Runs forever: we need all functions running simultaneously.
    vTaskResume(beepUP_handle);
    for (i = 0; i < ledCnt; i++) {
      matrix.drawPixel(i, 0, aC[i]);        // (int16_t x, int16_t y, uint16_t color);
      matrix.show();
      vTaskDelay( 50 / portTICK_PERIOD_MS );
      matrix.drawPixel(i, 0, BLACK);        // Blank out current pixel before next pixel is drawn.
      matrix.show();
    }
    vTaskDelay(250 / portTICK_PERIOD_MS );

    vTaskResume(beepDOWN_handle);
    // void marchRL() {   // Right-Left March. Display last RGYB LED briefly then retreat to previous RGYB LED, right to left.
    //beepDOWNsingle();
    //beepDOWN(NULL);
    for (i = ledCnt - 1; i >= 0; i--) {
      matrix.drawPixel(i, 0, aC[i]);
      matrix.show();
      vTaskDelay( 50 / portTICK_PERIOD_MS );
      matrix.drawPixel(i, 0, BLACK);  // Blank out current pixel before next pixel is drawn.
      matrix.show();
    }
    vTaskDelay(250 / portTICK_PERIOD_MS );
  }
}
//************//
void beepUP(void *pvParameters) {
  (void) pvParameters;
  while (1) {                    // Runs forever: we need all functions running simultaneously.
    for (j = 50; j <= 3000; j = j + 100) {
      tone(speakerPin, j, 50);  // tone(pin, frequency, duration)
      vTaskDelay(15 / portTICK_PERIOD_MS );
    }
    vTaskSuspend(beepUP_handle);
  }
}
//************//
void beepDOWN(void *pvParameters) {
  (void) pvParameters;
  while (1) {                    // Runs forever: we need all functions running simultaneously.
    for (int k = 3000; k >= 0; k = k - 100) {
      tone(speakerPin, k, 50);  // tone(pin, frequency, duration)
      vTaskDelay(15 / portTICK_PERIOD_MS );
    }
    vTaskSuspend(beepDOWN_handle);
  }
}
//************//
void swBlue(void *pvParameters) {       // When a push button switch is pressed and released, a blue LED illuminates.
  (void) pvParameters;                  // Pressing the switch a second time extinguishes the blue LED.
  while (1) {                           // Runs forever: we need all functions running simultaneously.
    pushNEW = digitalRead(pushButton);  // Read the pushbutton (switch is pulled high through 10K).
    //Serial.println(pushNEW);
    vTaskDelay(50 / portTICK_PERIOD_MS ); // Switch debounce.
    if (pushOLD == 1 && pushNEW == 0) {
      if (LEDstate == 0) {
        digitalWrite(blueLED, HIGH);      // Illuminate the blue LED.
        vTaskDelay( 50 / portTICK_PERIOD_MS );
        LEDstate = 1;
      } else {
        digitalWrite(blueLED, LOW);       // Extinguish the blue LED.
        LEDstate = 0;
        vTaskDelay( 500 / portTICK_PERIOD_MS );
      }
    }
    pushOLD = pushNEW;
  }
}
//************************************************************//
void beepUPsingle() {
  for (j = 50; j <= 3000; j = j + 100) {
    tone(speakerPin, j, 50);              // tone(pin, frequency, duration)
    vTaskDelay(20 / portTICK_PERIOD_MS );
  }
}
//************//
void beepDOWNsingle() {
  for (int k = 3000; k >= 0; k = k - 100) {
    tone(speakerPin, k, 50);  // tone(pin, frequency, duration)
    vTaskDelay(20 / portTICK_PERIOD_MS );
  }
}
//************//
void BEEPupDown(void *pvParameters) {
  (void) pvParameters;
  while (1) {                    // Runs forever: we need all functions running simultaneously.
    for (j = 50; j <= 3000; j = j + 100) {
      tone(speakerPin, j, 50);  // tone(pin, frequency, duration)
      vTaskDelay(100 / portTICK_PERIOD_MS );
    }
    for (int k = 3000; k >= 0; k = k - 100) {
      tone(speakerPin, k, 50);  // tone(pin, frequency, duration)
      vTaskDelay(100 / portTICK_PERIOD_MS );
    }
    j = 50; k = 50;
  }
}
//***********************************************************************//
void loop() { }
//***********************************************************************//

/* Place within a task: Delete, Suspend, Resume tasks.
   vTaskDelete(task handle);   // Once deleted it cannot be started again. Use Suspend and Resume instead.
   vTaskSuspend(task handle);
   vTaskResume(task handle);
*/
/* More accurate delay function:
   In global variables section, put:
    TickType_t  getTick;
    getTick = xTaskGetTickCount();    // getTick will get time from systick of OS.
   In tasks, put:
    vTaskDelayUntil(&getTick,1000 / portTICK_PERIOD_MS);.
*/
/*
  There are 6 arguments that should be passed while creating any task:
  - pvTaskCode: It is simply a pointer to the function that implements the task (in effect, just the name of the function).
  - pcName: A descriptive name for the task. This is not used by FreeRTOS. It is included purely for debugging purposes.
  - usStackDepth: Each task has its own unique stack that is allocated by the kernel to the task when the task is created.
    The value specifies the number of words the stack can hold, not the number of bytes. For example, if the stack is
    32-bits wide and usStackDepth is passed in as 100, then 400 bytes of stack space will be allocated (100 * 4 bytes) in RAM.
    Use this wisely because Arduino Uno has only 2Kbytes of RAM.
  - pvParameters: Task input parameter (can be NULL).
  - uxPriority: Priority of the task ( 0 is the lowest priority).
  - pxCreatedTask: It can be used to pass out a handle to the task being created. This handle can then be used to reference
    the task in API calls that, for example, change the task priority or delete the task (can be NULL).
*/
